
//    Fecha: 17/05/09
//    Autor: Adrin Sergio Buratovich
// Objetivo: Funcionalidad bsica para el ordenamiento por
//           columnas de los items de un objeto GRID

// NOTA 01 : No valid formatos del contenido por lo que en los
//           campos del tipo fecha se producir un ordenamiento
//           erroneo.
//           PENDIENTE:
//           01) Validar tipos desde datos del tipo CARACTER
//           02) Convertir datos de '01/05/09' a '20090501' en
//               la generacin del ndice
//           03) Convertir datos de '20090501' a '01/05/09' en
//               la transferencia al cntenido del GRID
//
// NOTA 02 : Inclu la funcin cStrToken() en reemplazo del uso
//           de strtoken() ya que sta ltima produce errores en
//           ciertos casos.
//           EJEMPLO:
//
//           StrToken('|2|3|4',1,'|') DEVUELVE '2'
//           cStrToken('|2|3|4',1,'|') DEVUELVE ''



//--------------------------------------------------------------//

FUNCTION GRIDSETORDER(_nGridCol)

*-------------------
 Local aDataGrid   := {}
 Local nColGrid    := 0
 Local nRowGrid    := 0
*-------------------
 Local cContOrd    := ""
 Local aContOrd    := {}
 Local aGridIndex  := {}
*-------------------
 Local cSeparador  := "|"
*-------------------

 If valtype(_nGridCol)<>"N"
  MsgStop("Error de tipo de dato de orden requerido.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf

 If valtype(this.Item(_nGridCol))<>"A"
  MsgStop("Error de tipo de contenidos a ser procesados.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf

 // Determinacin de parmetros del proceso
 aDataGrid := AClone(this.Item(_nGridCol))
 nColGrid  := Len(aDataGrid)
 nRowGrid  := this.ItemCount
 aContOrd  := Array(nColGrid)

 // Generacin de contenidos a ordenar
 For nRow := 1 To nRowGrid
  // Se asigna el contenido de la columna requerida
  cContOrd := this.Item(nRow)[_nGridCol] + cSeparador
  For nCol := 1 To nColGrid
   // Se asigna el contenido de las columnas restantes
   if nCol<>_nGridCol
    cContOrd := cContOrd + this.Item(nRow)[nCol] + cSeparador
   endif
  Next
  Aadd(aGridIndex,cContOrd)
 Next

 // Ordenamiento del contenido
 aGridIndex := Asort(aGridIndex)

 // Transferencia del contenido ordenado
 For nRow := 1 To nRowGrid

  cContOrd := aGridIndex[nRow]
  // Se transfiere el contenido de la columna requerida obteniendo
  // la posicin inicial del contenido ordenado
  nPosCont := 1
  aContOrd[_nGridCol] := cStrToken(cContOrd,nPosCont,cSeparador)
  // Se transfiere el contenido de las columnas restantes obteniendo
  // las posiciones del contenido ordenado, posteriores a la inicial
  // nPosCont
  For nCol := 1 To nColGrid
   // Slo son actualizadas las columnas diferentes a la requerida
   if nCol<>_nGridCol
    nPosCont++
    aContOrd[nCol] := cStrToken(cContOrd,nPosCont,cSeparador)
   endif
  Next

  // Se transfiere el contenido ordenado al control GRID
  this.Item(nRow) := aContOrd

 Next

 // Actualizacin de interfaz
 this.Refresh()

RETURN(.T.)

//--------------------------------------------------------------//

FUNCTION cStrToken( _cCadena, _nPosDato, _cSeparador )

// REEMPLAZA EL USO DE STRTOKEN YA QUE PRODUCE ERRORES
// CUANDO EXISTEN POSICIONES INICIALES SIN DATOS
//
// EJEMPLO:
//
//  StrToken('|2|3|4',1,'|') DEVUELVE '2'
// cStrToken('|2|3|4',1,'|') DEVUELVE ''

*------------------------
 Local nByte            := 0
 Local nBytes           := Len(_cCadena)
 Local nSeparador       := 0
*------------------------
 Local cByte            := ""
 Local cDato            := ""
*------------------------

 DEFAULT _cSeparador := '|'
 DEFAULT _nPosDato   := 1

 For nByte := 1 To nBytes

  cByte  := SubStr(_cCadena,nByte,1)

  If cByte == _cSeparador
   nSeparador++
   if nSeparador == _nPosDato
     RETURN( cDato )
   else
    cByte := ""
    cDato := ""
   endif
  EndIf

  cDato := cDato + cByte

 Next

 If nSeparador < ( _nPosDato - 1 )
  cDato := ""
 EndIf

RETURN( cDato )

//----------------------------------------------------------------------//